import configparser
import os
from pathlib import Path

class Config():
    def __init__(self, location):
        self.__cfile = location / "config.ini"
        self.__standardcfgs = ["tf_path"]
        self.__config = configparser.ConfigParser(allow_no_value = True)
        self.__bad_path = False
        
        if not(os.path.exists(self.__cfile)):
            try:
                vproject = Path(os.environ["VProject"])
            except KeyError:
                if not(os.path.exists(vproject / "tf2_misc_dir.vpk")):
                    self.__bad_path = not(os.path.exists(Path("C:\Program Files (x86)\Steam\steamapps\common\Team Fortress 2\tf")))
                    vproject = Path("C:\Program Files (x86)\Steam\steamapps\common\Team Fortress 2\tf")
        
            self.__config.add_section("Core")
            self.__config.set("Core", "; -- The TF path is set to the vproject dir by default. Change this to set it to somewhere else")
            self.__config.set("Core", "tf_path", str(vproject))
            self.__config.set("Core", "; -- If a file's extension is present here, it will not be listed when using -parse")
            self.__config.set("Core", "parse_ignore_ext", str([".vvd", ".vtx", ".phy", ".vtf"]))

            self.__writecf()
            print("No config file found at %s, creating a new one" %self.__cfile)

            if self.__bad_path:
                raise ValueError("Bad /tf/ folder path set. Open config.ini and point it to the correct location.")

    def __writecf(self):
        with open(self.__cfile, "w") as cf:
                self.__config.write(cf)

    def __readcf(self):
        self.__config.read(self.__cfile)

    def getkv(self, kv):
        self.__readcf()
        return self.__config["Core"][kv]

