from .lumps.structure_parser import Struct
from .vtxfile import VTXparser
from pathlib import Path
import os


class MDLparser():
    def __init__(self, tf_path, filepath):
        self.__MDLheader = Struct("mdl_file",
                            (("id", "IDST"),
                             ("version", "INTUL"),
                             ("checksum", "INTUL"), #should be same as in vtx and phy
                             ("name", "STR/64"),
                             ("dataLength", "INTUL"),
                             ("filler1", "RAW/72"), #data not needed
                             ("flags", "INTUL"),
                             ("filler2", "RAW/48"), #more data not needed
                             ("texture_count", "INTUL"), ("texture_offset", "INTUL"), #material filenames here
                             ("texturedir_count", "INTUL"), ("texturedir_offset", "INTUL"),
                             ("skinreference_count", "INTUL"), ("skinrfamily_count", "INTUL"),
                             ("skinreference_index", "INTUL"),
                             ("filler3", "RAW/216"))) #misc data until end of struct
                            ##408 bytes long

        self.__MDLmaterial = Struct("mdl_material",
                                     (("name_offset", "INTUL"), #num bytes past this structure's beginning to the name
                                      ("flags", "INTUL"),
                                      ("used", "INTUL"),
                                      ("unused", "INTUL"),
                                      ("material", "INTUL"),
                                      ("client_material", "INTUL"),
                                      ("unused2", "RAW/40")))
                                ##64 bytes long
        self.__MDLmaterial_final = Struct("mdl_material_fin",
                                              (("mdl_material", "mdl_material/1"),
                                              ("final_offset", "INTUL")),
                                              self.__MDLmaterial)

        self.__filepath = filepath
        self.__TF_PATH = tf_path

    def get_materials(self, args):
        with open(self.__filepath, "rb") as fp:
            data = self.__MDLheader.parse(fp)
            fp.seek(data["texture_offset"])
            offset_arr = []

            for repeat in range(data["texture_count"]-1):
                offset_arr.append(data["texture_offset"] + self.__MDLmaterial.parse(fp)["name_offset"] + 64*repeat)
            final_item = self.__MDLmaterial_final.parse(fp)
            offset_arr.append(data["texture_offset"] + final_item["mdl_material"][0]["name_offset"] + 64*(data["texture_count"]-1))
            offset_arr.append(final_item["final_offset"])

            output_fnames = []
            fp.seek(final_item["final_offset"])
            read_data = fp.read(data["dataLength"] - final_item["final_offset"] - 1).decode().strip("\x00")
            for item in read_data.split("\x00"):
                path = "materials\\" + item
                for x in range(len(offset_arr)-1):       
                    fp.seek(offset_arr[x])
                    if offset_arr[x] > offset_arr[x+1]:
                        readlen = offset_arr[x] - offset_arr[x+1] - 1
                    else:
                        readlen = offset_arr[x+1] - offset_arr[x] - 1
                    output_fnames.append(path + fp.read(readlen).decode() + ".vmt")


            ##fetch skins
            ##args has a list of dictionaries allowing the lookup of the model name to fetch skins for it
            fp.seek(data["skinreference_index"])
            sref_size = 2*data["skinreference_count"]*data["skinrfamily_count"]
            ref_data = fp.read(sref_size)
            
            used_ids = set()
            relative_fpath = str(self.__filepath.relative_to(self.__TF_PATH)).replace("\\", "/")
            for item in args:
                if relative_fpath.encode() in item: ##if valid skin(*), add it
                    used_ids.update(item[relative_fpath.encode()])

            for ext in [".sw.vtx", ".dx90.vtx", ".dx80.vtx"]: ##we use this to parse ported models with no .sw.vtx file
                vtxpath = self.__filepath.with_suffix(ext)
                if os.name == "posix":
                    for asset in (Path(asset) for asset in vtxpath.parent.iterdir() if asset.is_file() and asset.suffix == ".vtx"):
                        ##uppercase check
                        if (str(asset.name).lower()) != str(asset.name) and \
                                    asset.name.lower() == vtxpath.name.lower():
                            vtxpath = asset
                            break
                if not os.path.exists(vtxpath):
                    print("VTX file %s not found" %vtxpath)
                else:
                    break                    
            else:
                raise IOError("No .vtx files found for model %s" %self.__filepath)
            vtxparser = VTXparser(self.__TF_PATH, vtxpath)
            nmaterials = vtxparser.get_nmaterials()


            used_indices = set() ##index into material output array
            if used_ids == set():
                used_indices = {0,}
            for skinid in used_ids:
                if skinid >= data["skinreference_count"]:
                    print("Skin number found that is too large for model")
                    skinid = 0
                    ##bypass for if a skinid exists for a skin that doesn't
                    ###unsure if this is a proper fix. A more in-depth investigation 
                    ###with affected bsps (e.g. cp_overgrown) needs to be done.
                for x in range(nmaterials): ##for multi-material models, this will add to the index for every additional
                    used_indices.add(ref_data[skinid*2*data["skinreference_count"] + x*2]) ##check the *2's validity
        
            output_fnames_fixed = list(item for item in output_fnames if output_fnames.index(item) in used_indices)
                    
            return list(map(Path, output_fnames_fixed))
